// TokenizerClass.h

#ifndef _TOKENIZER_H
#define _TOKENIZER_H


#include <tchar.h>

#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

/////////////////////////////////////////////////////////////////////////////
// CTokenizer
//

/* 
   Usage...

   CTokenizer tokenizer("TokenFirst:TokenSecond TokenThird-TokenLast", ": -");

   printf("We have %d tokens\r\n", tokenizer.NumberOfTokens());

   for (int i=0; i<tokenizer.NumberOfTokens(); i++)
   printf("token %d is %s\r\n",i,tokenizer.ReturnToken(i));

   if (tokenizer.CheckForString("TokenFirst"))
   printf("TokenFirst is one of the tokens\r\n");

   ----------
   Output:

   We have 4 tokens
   token 0 is TokenFirst
   token 1 is TokenSecond
   token 2 is TokenThird
   token 3 is TokenLast
   TokenFirst is one of the tokens

   Notes:
   All tokens cease to exist with the object.
   BAD CODE - DO NOT DO:
      void MyBadProc(char * my_bad_ptr)
      {
         CTokenizer tok("bla bla", " ");
         my_bad_ptr = tok[1];
      }
      my_bad_ptr will end up pointing to deallocated mem


   END OF STRING gets counted as a delimiter, i.e., 
      CTokeinzer tok("there_is_no_delimiter_here", " ");
      will return 1 from NumberOfTokens even tho the delimiter is not found in the string

   The default behavior is to NOT count successive delimiters as a token, i.e., "1,2,,3" 
   is considered to contain 3 tokens, not 4. This behavior is changed by setting 
   bAllowEmptyTokens to true in the constructor. In the example above, when the third 
   token is retrieved using ReturnToken(2), the empty string _T("") is returned. 
   This kind of behavior would allow reading for example a csv record with empty fields. 
*/


class DECL CTokenizer
{
   public:
   // Constructor allocates mem for each token string.
   CTokenizer(const TCHAR * str_to_tokenize, const TCHAR * str_delimiters, bool bAllowEmptyTokens = false);
   ~CTokenizer();

   // Returns the specified token. Note: ReturnToken's int token param ranges from 
   // 0 to < number_of_tokens.
   TCHAR * ReturnToken(int token);
   TCHAR * operator[](int token) { return ReturnToken(token); };
   
   // Returns number of tokens in string.
   int NumberOfTokens(void) {return number_of_tokens;};
   
   // Check to see if passed string matches any of the token strings.
   BOOL CheckForString(const TCHAR * str_to_match);

   // Method to test class in debug build. 
   #ifdef _DEBUG
   void Test();
   #endif

   private:
   int CountTokens(const TCHAR * str_to_tokenize, const TCHAR * str_delimiters);
   bool IsDelimiter(const TCHAR ch_to_compare, const TCHAR * str_delimiters);
   void Tokenize(TCHAR * str_to_tokenize, const TCHAR * str_delimiters);

   private:
   bool m_bAllowEmptyTokens;
   int number_of_tokens;
   TCHAR ** ptr_token_array;
};


#endif // _TOKENIZER_H
